@@ -1,21 +1,34 @@
-from __future__ import absolute_import, division, unicode_literals
 import decimal
-import unittest
+from hashlib import md5
 from warnings import warn
 
-import jsonpickle
-from jsonpickle.compat import PY2
-from jsonpickle.compat import PY3
-
+import pytest
 from helper import SkippableTest
 
+import jsonpickle
+import jsonpickle.ext.yaml
 
-class Thing(object):
+
+class Thing:
     def __init__(self, name):
         self.name = name
         self.child = None
 
 
+class A:
+    def __init__(self):
+        self.id = md5(str(id(self)).encode()).hexdigest()[:5]  # unique enough hash
+
+
+class BSlots:
+    __slots__ = ["a2", "a1", "a3"]
+
+    def __init__(self):
+        self.a2 = A()  # set attribs not in alphabetical order
+        self.a1 = A()
+        self.a3 = self.a1  # create a reference
+
+
 SAMPLE_DATA = {'things': [Thing('data')]}
 
 
@@ -40,16 +53,16 @@ class BackendBase(SkippableTest):
         # always reset to default backend
         jsonpickle.set_preferred_backend('json')
 
-    def assertEncodeDecode(self, json_input):
+    def assert_roundtrip(self, json_input):
         expect = SAMPLE_DATA
         actual = jsonpickle.decode(json_input)
-        self.assertEqual(expect['things'][0].name, actual['things'][0].name)
-        self.assertEqual(expect['things'][0].child, actual['things'][0].child)
+        assert expect['things'][0].name == actual['things'][0].name
+        assert expect['things'][0].child == actual['things'][0].child
 
         pickled = jsonpickle.encode(SAMPLE_DATA)
         actual = jsonpickle.decode(pickled)
-        self.assertEqual(expect['things'][0].name, actual['things'][0].name)
-        self.assertEqual(expect['things'][0].child, actual['things'][0].child)
+        assert expect['things'][0].name == actual['things'][0].name
+        assert expect['things'][0].child == actual['things'][0].child
 
     def test_None_dict_key(self):
         """Ensure that backends produce the same result for None dict keys"""
@@ -57,7 +70,7 @@ class BackendBase(SkippableTest):
         expect = {'null': None}
         pickle = jsonpickle.encode(data)
         actual = jsonpickle.decode(pickle)
-        self.assertEqual(expect, actual)
+        assert expect == actual
 
     def test_encode_with_indent_and_separators(self):
         obj = {
@@ -66,7 +79,7 @@ class BackendBase(SkippableTest):
         }
         expect = '{\n' '    "a": 1,\n' '    "b": 2\n' '}'
         actual = jsonpickle.encode(obj, indent=4, separators=(',', ': '))
-        self.assertEqual(expect, actual)
+        assert expect == actual
 
 
 class JsonTestCase(BackendBase):
@@ -81,7 +94,7 @@ class JsonTestCase(BackendBase):
             '"child": null} '
             ']}'
         )
-        self.assertEncodeDecode(expected_pickled)
+        self.assert_roundtrip(expected_pickled)
 
 
 class SimpleJsonTestCase(BackendBase):
@@ -96,15 +109,15 @@ class SimpleJsonTestCase(BackendBase):
             '"child": null}'
             ']}'
         )
-        self.assertEncodeDecode(expected_pickled)
+        self.assert_roundtrip(expected_pickled)
 
     def test_decimal(self):
         # Default behavior: Decimal is preserved
         obj = decimal.Decimal(0.5)
         as_json = jsonpickle.dumps(obj)
         clone = jsonpickle.loads(as_json)
-        self.assertTrue(isinstance(clone, decimal.Decimal))
-        self.assertEqual(obj, clone)
+        assert isinstance(clone, decimal.Decimal)
+        assert obj == clone
 
         # Custom behavior: we want to use simplejson's Decimal support.
         jsonpickle.set_encoder_options('simplejson', use_decimal=True, sort_keys=True)
@@ -114,16 +127,27 @@ class SimpleJsonTestCase(BackendBase):
         # use_decimal mode allows Decimal objects to pass-through to simplejson.
         # The end result is we get a simple '0.5' value as our json string.
         as_json = jsonpickle.dumps(obj, unpicklable=True, use_decimal=True)
-        self.assertEqual(as_json, '0.5')
+        assert as_json == '0.5'
         # But when loading we get back a Decimal.
         clone = jsonpickle.loads(as_json)
-        self.assertTrue(isinstance(clone, decimal.Decimal))
+        assert isinstance(clone, decimal.Decimal)
 
         # side-effect: floats become decimals too!
         obj = 0.5
         as_json = jsonpickle.dumps(obj)
         clone = jsonpickle.loads(as_json)
-        self.assertTrue(isinstance(clone, decimal.Decimal))
+        assert isinstance(clone, decimal.Decimal)
+        # options are persisted unless we disable them
+        jsonpickle.set_encoder_options('simplejson', use_decimal=False)
+        jsonpickle.set_decoder_options('simplejson', use_decimal=False)
+
+    def test_sort_keys(self):
+        jsonpickle.set_encoder_options('simplejson', sort_keys=True)
+        b = BSlots()
+        with pytest.raises(TypeError):
+            jsonpickle.encode(b, keys=True, warn=True)
+        # return encoder options to default
+        jsonpickle.set_encoder_options('simplejson', sort_keys=False)
 
 
 def has_module(module):
@@ -135,37 +159,6 @@ def has_module(module):
     return True
 
 
-class JsonlibTestCase(BackendBase):
-    def setUp(self):
-        if PY2:
-            self.set_preferred_backend('jsonlib')
-
-    def test_backend(self):
-        if PY3:
-            return self.skip('no jsonlib for python3')
-        expected_pickled = (
-            '{"things":[{'
-            r'"py\/object":"backend_test.Thing",'
-            '"name":"data","child":null}'
-            ']}'
-        )
-        self.assertEncodeDecode(expected_pickled)
-
-
-class YajlTestCase(BackendBase):
-    def setUp(self):
-        self.set_preferred_backend('yajl')
-
-    def test_backend(self):
-        expected_pickled = (
-            '{"things":[{'
-            '"py/object":"backend_test.Thing",'
-            '"name":"data","child":null}'
-            ']}'
-        )
-        self.assertEncodeDecode(expected_pickled)
-
-
 class UJsonTestCase(BackendBase):
     def setUp(self):
         self.set_preferred_backend('ujson')
@@ -177,21 +170,19 @@ class UJsonTestCase(BackendBase):
             '"name":"data","child":null}'
             ']}'
         )
-        self.assertEncodeDecode(expected_pickled)
+        self.assert_roundtrip(expected_pickled)
 
 
-def suite():
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(JsonTestCase))
-    suite.addTest(unittest.makeSuite(UJsonTestCase))
-    suite.addTest(unittest.makeSuite(SimpleJsonTestCase))
-    if has_module('yajl'):
-        suite.addTest(unittest.makeSuite(YajlTestCase))
-    if PY2:
-        if has_module('jsonlib'):
-            suite.addTest(unittest.makeSuite(JsonlibTestCase))
-    return suite
+class YamlTestCase(BackendBase):
+    def setUp(self):
+        jsonpickle.ext.yaml.register()
+        self.set_preferred_backend('yaml')
 
-
-if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    def test_backend(self):
+        expected_pickled = (
+            'things:\n'
+            '  - py/object: backend_test.Thing\n'
+            '    name: data\n'
+            '    child: null\n'
+        )
+        self.assert_roundtrip(expected_pickled)
