@@ -6,7 +6,6 @@ import pytest
 try:
     import numpy as np
     import numpy.testing as npt
-    from numpy.compat import asbytes
     from numpy.testing import assert_equal
 except ImportError:
     pytest.skip('numpy is not available', allow_module_level=True)
@@ -19,7 +18,7 @@ from jsonpickle import handlers
 @pytest.fixture(scope='module', autouse=True)
 def numpy_extension():
     """Initialize the numpy extension for this test module"""
-    jsonpickle.ext.numpy.register_handlers()
+    jsonpickle.ext.numpy.register_handlers(ndarray_mode='warn')
     yield  # control to the test function.
     jsonpickle.ext.numpy.unregister_handlers()
 
@@ -32,13 +31,10 @@ def test_dtype_roundtrip():
     dtypes = [
         np.int_,
         np.int32,
-        np.float_,
         np.float64,
-        np.complex_,
         np.complex128,
         np.str_,
         np.object_,
-        np.compat.unicode,
         np.dtype(np.void),
         np.dtype(np.int32),
         np.dtype(np.float32),
@@ -86,15 +82,14 @@ def test_generic_roundtrip():
     values = [
         np.int_(1),
         np.int32(-2),
-        np.float_(2.5),
+        np.float64(2.5),
         np.nan,
         -np.inf,
         np.inf,
         np.datetime64('2014-01-01'),
         np.str_('foo'),
-        np.unicode_('bar'),
         np.object_({'a': 'b'}),
-        np.complex_(1 - 2j),
+        np.complex128(1 - 2j),
     ]
     for value in values:
         decoded = roundtrip(value)
@@ -107,9 +102,9 @@ def test_ndarray_roundtrip():
         np.random.random((10, 20)),
         np.array([[True, False, True]]),
         np.array(['foo', 'bar']),
-        np.array(['baz'.encode('utf-8')]),
+        np.array([b'baz']),
         np.array(['2010', 'NaT', '2030']).astype('M'),
-        np.rec.array(asbytes('abcdefg') * 100, formats='i2,a3,i4', shape=3),
+        np.rec.array(b'abcdefg' * 100, formats='i2,a3,i4', shape=3),
         np.rec.array(
             [
                 (1, 11, 'a'),
@@ -264,7 +259,7 @@ def test_fortran_base():
 
 def test_buffer():
     """test behavior with memoryviews which are not ndarrays"""
-    bstring = 'abcdefgh'.encode('utf-8')
+    bstring = b'abcdefgh'
     a = np.frombuffer(bstring, dtype=np.byte)
     warn_count = 1
     with warnings.catch_warnings(record=True) as w:
@@ -309,18 +304,22 @@ def test_immutable():
 def test_byteorder():
     """Test the byteorder for text and binary encodings"""
     # small arr is stored as text
-    a = np.arange(10).newbyteorder()
-    b = a[:].newbyteorder()
-    _a, _b = roundtrip([a, b])
-    npt.assert_array_equal(a, _a)
-    npt.assert_array_equal(b, _b)
+    a = np.arange(10)
+    av = a.view(a.dtype.newbyteorder())
+    b = a[:]
+    bv = b.view(b.dtype.newbyteorder())
+    _av, _bv = roundtrip([av, bv])
+    npt.assert_array_equal(av, _av)
+    npt.assert_array_equal(bv, _bv)
 
     # bigger arr is stored as binary
-    a = np.arange(100).newbyteorder()
-    b = a[:].newbyteorder()
-    _a, _b = roundtrip([a, b])
-    npt.assert_array_equal(a, _a)
-    npt.assert_array_equal(b, _b)
+    a = np.arange(100)
+    av = a.view(a.dtype.newbyteorder())
+    b = a[:]
+    bv = b.view(b.dtype.newbyteorder())
+    _av, _bv = roundtrip([av, bv])
+    npt.assert_array_equal(av, _av)
+    npt.assert_array_equal(bv, _bv)
 
 
 def test_zero_dimensional_array():
@@ -376,5 +375,32 @@ def test_np_poly1d():
     assert obj == jsonpickle.decode(jsonpickle.encode(obj))
 
 
-if __name__ == '__main__':
-    pytest.main([__file__])
+def test_np_datetime64_units():
+    """Ensure that we can roundtrip a numpy datetime64 with varying precisions"""
+    obj = np.datetime64('2000', 'ns')
+    encoded = jsonpickle.encode(obj)
+    assert obj == jsonpickle.decode(encoded)
+
+    obj = np.datetime64('2001', 'ms')
+    encoded = jsonpickle.encode(obj)
+    assert obj == jsonpickle.decode(encoded)
+
+    obj = np.datetime64('2002', 's')
+    encoded = jsonpickle.encode(obj)
+    assert obj == jsonpickle.decode(encoded)
+
+    obj = np.datetime64('2003', 'm')
+    encoded = jsonpickle.encode(obj)
+    assert obj == jsonpickle.decode(encoded)
+
+    obj = np.datetime64('2004', 'h')
+    encoded = jsonpickle.encode(obj)
+    assert obj == jsonpickle.decode(encoded)
+
+    obj = np.datetime64('2005', 'D')
+    encoded = jsonpickle.encode(obj)
+    assert obj == jsonpickle.decode(encoded)
+
+    obj = np.datetime64('2006', 'Y')
+    encoded = jsonpickle.encode(obj)
+    assert obj == jsonpickle.decode(encoded)
